#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Collections;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum anaPlotAlignPD38 {Left, Right, DoNotPlot}
public enum anaPivotStylesPD38 {Floor, Wide, Fibonacci, Jackson, Camarilla, Woodie}
public enum anaSessionTypePD38 {ETH, RTH, Auto}
public enum anaSessionCountPD38 {First, Second, Third}

#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Pivot Points.
	/// </summary>
	[Description("Daily Session Pivots.")]
	public class anaPivotsDailyV38: Indicator
	{
		#region Variables
		private	SolidBrush[]		brushes					= { new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),  
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),  
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black)};
		private DateTime 			sessionBegin			= Cbi.Globals.MinDate;
		private DateTime 			sessionEnd 				= Cbi.Globals.MinDate;
		private DateTime			plotStart				= Cbi.Globals.MaxDate;
		private DateTime			cacheSessionBeginTmp   	= Cbi.Globals.MinDate;
		private DateTime			cacheSessionEndTmp		= Cbi.Globals.MinDate;
		private DateTime			cacheSessionDate		= Cbi.Globals.MinDate;
		private DateTime			currentDate				= Cbi.Globals.MinDate;
		private DateTime			priorDate				= Cbi.Globals.MinDate;
		private	double				currentHigh				= double.MinValue;
		private	double				currentLow				= double.MaxValue;
		private	double				currentClose			= 0.0;
		private	double				volSum					= 0.0;
		private	double				volPriceSum				= 0.0;
		private	double				currentVolSum			= 0.0;
		private	double				currentVolPriceSum		= 0.0;
		private double				displaySize				= 0.0;
		private Bars				dailyBars				= null;
		private SolidBrush			textBrush				= new SolidBrush(Color.Red);
		private Font				textFont				= new Font("Arial", 12);
		private Font				labelFont				= new Font("Arial", 8);
		private string				errorData1				= "Daily Pivots can only be displayed on intraday charts.";
		private string				errorData2				= "No daily data found. Please reload daily data or use CalcFromIntradayData.";
		private string				errorData3				= "Loading of daily data is incomplete. Please reload daily data or use CalcFromIntradayData.";
		private	string				errorData4				= "Insufficient historical data to calculate pivots. Please increase chart look back period.";
		private	string				errorData5				= "The prior close is not within the range of the selected RTH session.";
        private float				errorTextWidth			= 0;
        private float				errorTextHeight			= 0;
		private bool				existsHistDailyData		= false;
		private bool				isDailyDataLoaded		= false;
		private bool				plotPivots				= true;
		private bool				periodOpen				= false;
		private bool				bridgePublicHoliday		= false; 
		private bool				extendPublicHoliday		= false; 
		private bool				noDailyData				= false;
		private bool				noCurrentDailyBar		= false;
		private bool				isCurrency				= false;
		private bool				isGlobex				= false;
		private ArrayList			newSessionBarIdxArr1	= new ArrayList();
		private anaPlotAlignPD38	plotLabels				= anaPlotAlignPD38.Right;
		private anaPivotStylesPD38 	pivotFormula			= anaPivotStylesPD38.Floor;
		private anaSessionTypePD38	pivotSession			= anaSessionTypePD38.Auto;
		private anaSessionCountPD38 activeSession			= anaSessionCountPD38.Second;
		private Data.PivotRange		pivotRangeType1			= PivotRange.Daily;
		private HLCCalculationMode	priorDayHLC				= HLCCalculationMode.DailyBars;
		private double				range					= 0.0;
		private	double				pp						= 0.0;
		private	double				r1						= 0.0;
		private	double				r2						= 0.0;
		private	double				r3						= 0.0;
		private	double				r4						= 0.0;
		private	double				r5						= 0.0;
		private	double				s1						= 0.0;
		private	double				s2						= 0.0;
		private	double				s3						= 0.0;
		private	double				s4						= 0.0;
		private	double				s5						= 0.0;
		private	double				rmid					= 0.0;
		private	double				r12mid					= 0.0;
		private	double				r23mid					= 0.0;
		private	double				r34mid					= 0.0;
		private	double				r45mid					= 0.0;
		private	double				smid					= 0.0;
		private	double				s12mid					= 0.0;
		private	double				s23mid					= 0.0;
		private	double				s34mid					= 0.0;
		private	double				s45mid					= 0.0;
		private double				centralPivot			= 0.0;
		private double				directionalPivot		= 0.0;
		private double				previousVWAP			= 0.0;
		private double				previousHigh			= 0.0;
		private double				previousLow				= 0.0;
		private double				previousClose			= 0.0;
		private DateTime			sessionDateTmp			= Cbi.Globals.MinDate;
		private DateTime			sessionDateDaily		= Cbi.Globals.MinDate;
		private	StringFormat		stringFormatCenter		= new StringFormat();
		private	StringFormat		stringFormatFar			= new StringFormat();
		private	StringFormat		stringFormatNear		= new StringFormat();
		private double				userDefinedClose		= 0.0;
		private double				userDefinedHigh			= 0.0;
		private double				userDefinedLow			= 0.0;
		private int					countDown				= 1;
		private int					numberOfSessions		= 1;
		private int					sessionCount			= 0;
		private int					width					= 20;
		private int 				labelFontSize			= 8;
		private int 				labelOffset				= 20;
		private bool				rthPivots				= false;
		private bool				showMidpivots			= false;
		private int 				plot0Width 				= 1;
		private DashStyle 			dash0Style				= DashStyle.Solid;
		private int 				plot1Width 				= 1;
		private DashStyle 			dash1Style				= DashStyle.DashDot;
		private int 				plot2Width 				= 2;
		private DashStyle 			dash2Style				= DashStyle.Dot;
		private Color 				pivotColor				= Color.DeepSkyBlue;
		private Color 				resistanceColor 		= Color.DeepSkyBlue;
		private Color 				supportColor	 		= Color.DeepSkyBlue;
		private Color				midpivotColor			= Color.DeepSkyBlue;
		private Color				highColor				= Color.Lime;
		private Color				lowColor				= Color.OrangeRed;	
		private Color				closeColor				= Color.Yellow;
		private Color				centralPivotColor		= Color.MediumSpringGreen;
		private Color				directionalPivotColor	= Color.MediumSpringGreen;
		private Color				vwapColor				= Color.HotPink;
		private DateTime			publicHoliday0			= new DateTime (2009,01,19);
		private DateTime			publicHoliday1			= new DateTime (2009,02,16);
		private DateTime			publicHoliday2			= new DateTime (2009,05,25);
		private DateTime			publicHoliday3			= new DateTime (2009,07,03);
		private DateTime			publicHoliday4			= new DateTime (2009,09,07);
		private DateTime			publicHoliday5			= new DateTime (2009,11,26);
		private DateTime			publicHoliday6			= new DateTime (2010,01,18);
		private DateTime			publicHoliday7			= new DateTime (2010,02,15);
		private DateTime			publicHoliday8			= new DateTime (2010,05,31);
		private DateTime			publicHoliday9			= new DateTime (2010,07,05);
		private DateTime			publicHoliday10			= new DateTime (2010,09,06);
		private DateTime			publicHoliday11			= new DateTime (2010,11,25);
		private DateTime			publicHoliday12			= new DateTime (2011,01,17);
		private DateTime			publicHoliday13			= new DateTime (2011,02,21);
		private DateTime			publicHoliday14			= new DateTime (2011,05,30);
		private DateTime			publicHoliday15			= new DateTime (2011,07,04);
		private DateTime			publicHoliday16			= new DateTime (2011,09,05);
		private DateTime			publicHoliday17			= new DateTime (2011,11,24);
		private DateTime			publicHoliday18			= new DateTime (2012,01,16);
		private DateTime			publicHoliday19			= new DateTime (2012,02,20);
		private DateTime			publicHoliday20			= new DateTime (2012,05,28);
		private DateTime			publicHoliday21			= new DateTime (2012,07,04);
		private DateTime			publicHoliday22			= new DateTime (2012,09,03);
		private DateTime			publicHoliday23			= new DateTime (2012,11,22);
		private DateTime[]			publicHoliday			= new DateTime [24];
		
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"PP "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"R1 "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"S1 "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"R2 "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"S2 "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"R3 "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"S3 "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"R4 "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"S4 "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"R5 "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"S5 "));
			
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"RMid "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"SMid "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"R12Mid "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"S12Mid "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"R23Mid "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"S23Mid "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"R34Mid "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"S34Mid "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"R45Mid "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"S45Mid "));
			
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"Y-High "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"Y-Low "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"Y-Close "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"CP "));
			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"DP "));

			Add(new Plot(new Pen(Color.Gray,1), PlotStyle.Line,"Y-VWAP"));

			AutoScale						= false;
			Overlay							= true;
			PriceTypeSupported				= false;
			PlotsConfigurable				= false;
			ZOrder							= 1;
			stringFormatNear.Alignment 		= StringAlignment.Near;
			stringFormatCenter.Alignment 	= StringAlignment.Center;
			stringFormatFar.Alignment		= StringAlignment.Far;
		}

		/// <summary>
		/// </summary>
		protected override void OnStartUp()
		{
			if (Instrument.MasterInstrument.InstrumentType == InstrumentType.Future &&
				(Instrument.MasterInstrument.Name == "EMD" ||Instrument.MasterInstrument.Name == "ES" || Instrument.MasterInstrument.Name == "NQ" 
				||Instrument.MasterInstrument.Name == "YM"||Instrument.MasterInstrument.Name == "GE" ||Instrument.MasterInstrument.Name == "SR" 
				||Instrument.MasterInstrument.Name == "UB"||Instrument.MasterInstrument.Name == "ZB" ||Instrument.MasterInstrument.Name == "ZF"
				||Instrument.MasterInstrument.Name == "ZN"||Instrument.MasterInstrument.Name == "ZQ" ||Instrument.MasterInstrument.Name == "ZT"
				||Instrument.MasterInstrument.Name == "6A"||Instrument.MasterInstrument.Name == "6B" ||Instrument.MasterInstrument.Name == "6C"
				||Instrument.MasterInstrument.Name == "6E"||Instrument.MasterInstrument.Name == "6J" ||Instrument.MasterInstrument.Name == "6M"
				||Instrument.MasterInstrument.Name == "6N"||Instrument.MasterInstrument.Name == "6S" ||Instrument.MasterInstrument.Name == "E7"
				||Instrument.MasterInstrument.Name == "J7"||Instrument.MasterInstrument.Name == "M6A" ||Instrument.MasterInstrument.Name == "M6B" 
				||Instrument.MasterInstrument.Name == "M6C"||Instrument.MasterInstrument.Name == "M6E" ||Instrument.MasterInstrument.Name == "M6J"
				||Instrument.MasterInstrument.Name == "M6S"||Instrument.MasterInstrument.Name == "CL" ||Instrument.MasterInstrument.Name == "EH"
				||Instrument.MasterInstrument.Name == "GC"||Instrument.MasterInstrument.Name == "HG" ||Instrument.MasterInstrument.Name == "HO"
				||Instrument.MasterInstrument.Name == "NG"||Instrument.MasterInstrument.Name == "QG" ||Instrument.MasterInstrument.Name == "QM"
				||Instrument.MasterInstrument.Name == "RB"||Instrument.MasterInstrument.Name == "SI" ||Instrument.MasterInstrument.Name == "YG" 
				||Instrument.MasterInstrument.Name == "YI"||Instrument.MasterInstrument.Name == "GF" ||Instrument.MasterInstrument.Name == "GPB"
				||Instrument.MasterInstrument.Name == "HE"||Instrument.MasterInstrument.Name == "LE" ||Instrument.MasterInstrument.Name == "YC"
				||Instrument.MasterInstrument.Name == "YK"||Instrument.MasterInstrument.Name == "YW" ||Instrument.MasterInstrument.Name == "ZC"
				||Instrument.MasterInstrument.Name == "ZE"||Instrument.MasterInstrument.Name == "ZL" ||Instrument.MasterInstrument.Name == "ZM"
				||Instrument.MasterInstrument.Name == "ZO"||Instrument.MasterInstrument.Name == "ZR" ||Instrument.MasterInstrument.Name == "ZS"
				||Instrument.MasterInstrument.Name == "ZW"))				
				isGlobex = true;
			if (isGlobex)
			{
				publicHoliday[0] = publicHoliday0;
				publicHoliday[1] = publicHoliday1;
				publicHoliday[2] = publicHoliday2;
				publicHoliday[3] = publicHoliday3;
				publicHoliday[4] = publicHoliday4;
				publicHoliday[5] = publicHoliday5;
				publicHoliday[6] = publicHoliday6;
				publicHoliday[7] = publicHoliday7;
				publicHoliday[8] = publicHoliday8;
				publicHoliday[9] = publicHoliday9;
				publicHoliday[10] = publicHoliday10;
				publicHoliday[11] = publicHoliday11;
				publicHoliday[12] = publicHoliday12;
				publicHoliday[13] = publicHoliday13;
				publicHoliday[14] = publicHoliday14;
				publicHoliday[15] = publicHoliday15;
				publicHoliday[16] = publicHoliday16;
				publicHoliday[17] = publicHoliday17;
				publicHoliday[18] = publicHoliday18;
				publicHoliday[19] = publicHoliday19;
				publicHoliday[20] = publicHoliday20;
				publicHoliday[21] = publicHoliday21;
				publicHoliday[22] = publicHoliday22;
				publicHoliday[23] = publicHoliday23;
			}			
			if (Instrument.MasterInstrument.InstrumentType == Cbi.InstrumentType.Currency || Instrument.MasterInstrument.Name == "DX"|| Instrument.MasterInstrument.Name == "6A"
				|| Instrument.MasterInstrument.Name == "6B" || Instrument.MasterInstrument.Name == "6C" ||Instrument.MasterInstrument.Name == "6E"
				|| Instrument.MasterInstrument.Name == "6J" || Instrument.MasterInstrument.Name == "6M" || Instrument.MasterInstrument.Name == "6S"
				|| Instrument.MasterInstrument.Name == "6N" || Instrument.MasterInstrument.Name == "E7" || Instrument.MasterInstrument.Name == "J7"
				|| Instrument.MasterInstrument.Name == "M6A" || Instrument.MasterInstrument.Name == "M6B" || Instrument.MasterInstrument.Name == "M6C"
				|| Instrument.MasterInstrument.Name == "M6E" || Instrument.MasterInstrument.Name == "M6J" || Instrument.MasterInstrument.Name == "M6S")
				isCurrency = true;
			
			if (pivotSession == anaSessionTypePD38.ETH || (pivotSession == anaSessionTypePD38.Auto && isCurrency))
				rthPivots = false;
			else
				rthPivots = true;
			
			labelFont = new Font ("Arial", labelFontSize);
			if (PivotFormula == anaPivotStylesPD38.Jackson)
			{
				Plots[0].Name = "BP";
				Plots[1].Name = "R1";
				Plots[2].Name = "S1";
				Plots[3].Name = "RB1";
				Plots[4].Name = "SB1";
				Plots[5].Name = "R2";
				Plots[6].Name = "S2";
				Plots[7].Name = "RB2";
				Plots[8].Name = "SB2";
				Plots[9].Name = "R3";
				Plots[10].Name = "S3";
			}
			else
			{
				Plots[0].Name = "PP";
				Plots[1].Name = "R1";
				Plots[2].Name = "S1";
				Plots[3].Name = "R2";
				Plots[4].Name = "S2";
				Plots[5].Name = "R3";
				Plots[6].Name = "S3";
				Plots[7].Name = "R4";
				Plots[8].Name = "S4";
				Plots[9].Name = "R5";
				Plots[10].Name = "S5";
			}
			Plots[0].Pen.Color = pivotColor;
			Plots[1].Pen.Color = resistanceColor;
			Plots[2].Pen.Color = supportColor;
			Plots[3].Pen.Color = resistanceColor;
			Plots[4].Pen.Color = supportColor;
			Plots[5].Pen.Color = resistanceColor;
			Plots[6].Pen.Color = supportColor;
			Plots[7].Pen.Color = resistanceColor;
			Plots[8].Pen.Color = supportColor;
			Plots[9].Pen.Color = resistanceColor;
			Plots[10].Pen.Color = supportColor;
			Plots[11].Pen.Color = midpivotColor;
			Plots[12].Pen.Color = midpivotColor;
			Plots[13].Pen.Color = midpivotColor;
			Plots[14].Pen.Color = midpivotColor;
			Plots[15].Pen.Color = midpivotColor;
			Plots[16].Pen.Color = midpivotColor;
			Plots[17].Pen.Color = midpivotColor;
			Plots[18].Pen.Color = midpivotColor;
			Plots[19].Pen.Color = midpivotColor;
			Plots[20].Pen.Color = midpivotColor;
			Plots[21].Pen.Color = highColor;
			Plots[22].Pen.Color = lowColor;
			Plots[23].Pen.Color = closeColor;
			Plots[24].Pen.Color = centralPivotColor;
			Plots[25].Pen.Color = directionalPivotColor;
			Plots[26].Pen.Color = vwapColor;
			
			for (int i = 0; i < 11; i++)
			{
				Plots[i].Pen.Width = plot0Width;
				Plots[i].Pen.DashStyle = dash0Style;
			}
			for (int i = 11; i < 21; i++)
			{
				Plots[i].Pen.Width = plot1Width;
				Plots[i].Pen.DashStyle = dash1Style;
			}
			for (int i = 21; i < 24; i++)
			{
				Plots[i].Pen.Width = plot2Width;
				Plots[i].Pen.DashStyle = dash2Style;
			}			
			for (int i = 24; i < 26; i++)
			{
				Plots[i].Pen.Width = plot0Width;
				Plots[i].Pen.DashStyle = dash0Style;
			}			
			if (AutoScale)
				AutoScale = false;
			if (Instrument.MasterInstrument.InstrumentType == Cbi.InstrumentType.Currency && (TickSize == 0.00001 || TickSize == 0.001))
				displaySize = 5* TickSize;
			else
				displaySize = TickSize;
			if(priorDayHLC == HLCCalculationMode.CalcFromIntradayData)
				countDown = 1;
			else
				countDown = 0;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (Bars == null)
				return; 
			if (!Bars.BarsType.IsIntraday && Bars.Period.Id != PeriodType.Day)
				return;
			if (Bars.Period.Id == PeriodType.Day && Bars.Period.Value > 1)
				return;
			if (Bars.Period.Id == PeriodType.Day && pivotRangeType1 == PivotRange.Daily)
			{
				textBrush.Color = ChartControl.AxisColor;
				DrawTextFixed("errortag1", errorData1, TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
				return;
			}			

			if (CurrentBar == 0)
			{
				if (priorDayHLC == HLCCalculationMode.UserDefinedValues)
				{
					currentHigh			= userDefinedHigh;
					currentLow			= userDefinedLow;
					currentClose		= userDefinedClose;
				}
				else
				{
					currentDate = GetLastBarSessionDate(Time[0], Bars, 0, pivotRangeType1);
					sessionCount = 1;
				}
			}
			if(!isDailyDataLoaded)
			{
				if (priorDayHLC == HLCCalculationMode.DailyBars && Bars.BarsType.IsIntraday) 
				{
					Enabled = false;
					System.Threading.ThreadPool.QueueUserWorkItem(new System.Threading.WaitCallback(GetBarsNow));
					return;
				}
				existsHistDailyData = false;
				isDailyDataLoaded	= true;
				if (priorDayHLC != HLCCalculationMode.UserDefinedValues)
					return;
			}
			if (priorDayHLC == HLCCalculationMode.DailyBars && !existsHistDailyData)
			{
				textBrush.Color = ChartControl.AxisColor;
				DrawTextFixed("errortag2", errorData2, TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
				return;
			}	

			IBar dailyBar;
			if (existsHistDailyData) 
			{
				noCurrentDailyBar = false;
				noDailyData = false;
				dailyBar = dailyBars.Get(dailyBars.GetBar(currentDate));
				if (dailyBar.Time.Date != currentDate)
				{
					noCurrentDailyBar = true;
					dailyBar = dailyBars.Get(dailyBars.GetBar(priorDate));
					if (dailyBar.Time.Date != priorDate)
					{
						noDailyData = true;
					}	
				}
			} 
			else 
				dailyBar = null;

			double	high		= (existsHistDailyData && !rthPivots) ? dailyBar.High : High[0];
			double	low			= (existsHistDailyData && !rthPivots) ? dailyBar.Low : Low[0];
			double	close		= existsHistDailyData ? dailyBar.Close : Close[0];

			DateTime lastBarTimeStamp1 = GetLastBarSessionDate(Time[0], Bars, 0, pivotRangeType1);
			if (lastBarTimeStamp1 != currentDate)
			{
				if (countDown == 0)
				{
					Bars.Session.GetNextBeginEnd(Bars, 0, out sessionBegin, out sessionEnd);
					plotStart = sessionBegin;
				}	
				countDown = countDown -1;
				sessionCount = 1;
				periodOpen = false;
				if (countDown < 0 && existsHistDailyData)
				{	
					currentClose = close;
					if (!rthPivots)
					{
						currentHigh = Math.Max(currentHigh, high);
						currentLow = Math.Min(currentLow, low);
					}
				}
				priorDate = currentDate;
				currentDate = lastBarTimeStamp1;
				bridgePublicHoliday = false;
				for (int i =0; i<24; i++)
				{
					if (publicHoliday[i].Date == priorDate)
						bridgePublicHoliday = true;
				}
				if ((countDown < 0 && !bridgePublicHoliday) || priorDayHLC == HLCCalculationMode.UserDefinedValues)
				{
					range = (currentHigh - currentLow);						
					if (PivotFormula == anaPivotStylesPD38.Fibonacci)
					{
						pp				= (currentHigh + currentLow + currentClose)/ 3;
						r1				= pp + 0.382*range;
						s1				= pp - 0.382*range;
						r2				= pp + 0.618*range;
						s2				= pp - 0.618*range;
						r3				= pp + range;
						s3				= pp - range;
						r4				= pp + 1.382*range;
						s4				= pp - 1.382*range;
						r5				= pp + 1.618*range;
						s5 				= pp - 1.618*range;
					}
					else if (PivotFormula == anaPivotStylesPD38.Jackson)
					{
						pp				= (currentHigh + currentLow + currentClose)/ 3;
						r1				= pp + 0.500*range;
						s1				= pp - 0.500*range;
						r2				= pp + 0.618*range;
						s2				= pp - 0.618*range;
						r3				= pp + range;
						s3				= pp - range;
						r4				= pp + 1.382*range;
						s4				= pp - 1.382*range;
						r5				= pp + 1.618*range;
						s5 				= pp - 1.618*range;
					}
					else if (PivotFormula == anaPivotStylesPD38.Camarilla)
					{
						pp				= (currentHigh + currentLow + currentClose)/ 3;
						r1				= currentClose + 1.1/12*range;
						s1				= currentClose - 1.1/12*range;
						r2				= currentClose + 1.1/6*range;
						s2				= currentClose - 1.1/6*range;
						r3				= currentClose + 1.1/4*range;
						s3				= currentClose - 1.1/4*range;
						r4				= currentClose + 1.1/2*range;
						s4				= currentClose - 1.1/2*range;
						if(currentLow != 0)
						{
							r5			= currentClose * currentHigh / currentLow; 
							s5 			= 2 * currentClose - r5;
						}
					}
					else
					{
						if (PivotFormula == anaPivotStylesPD38.Floor || PivotFormula == anaPivotStylesPD38.Wide)
							pp				= (currentHigh + currentLow + currentClose)/ 3;
						else if (PivotFormula == anaPivotStylesPD38.Woodie)
							pp				= (currentHigh + currentLow + 2*currentClose)/ 4;
						
						r1				= 2 * pp - currentLow;
						s1				= 2 * pp - currentHigh;
						r2				= pp + range;
						s2				= pp - range;
						if (PivotFormula == anaPivotStylesPD38.Floor || PivotFormula == anaPivotStylesPD38.Woodie)
						{
							r3				= r1 + range;
							s3				= s1 - range;
							r4				= r3 + (pp - currentLow);
							s4				= s3 - (currentHigh - pp);
							r5				= r4 + (pp - currentLow);
							s5				= s4 - (currentHigh - pp);
						}
						else if (PivotFormula == anaPivotStylesPD38.Wide)
						{
							r3				= r2 + range;
							s3				= s2 - range;
							r4				= r3 + range;
							s4				= s3 - range;
							r5				= r4 + range;
							s5				= s4 - range;
						}
						rmid			= 0.5*(pp+r1);
						smid			= 0.5*(pp+s1);
						r12mid			= 0.5*(r1+r2);
						s12mid			= 0.5*(s1+s2);
						r23mid			= 0.5*(r2+r3);
						s23mid			= 0.5*(s2+s3);
						r34mid			= 0.5*(r3+r4);
						s34mid			= 0.5*(s3+s4);
						r45mid			= 0.5*(r4+r5);
						s45mid			= 0.5*(s4+s5);
					}
					previousHigh		= currentHigh;
					previousLow			= currentLow;
					previousClose		= currentClose;
					centralPivot		= (currentHigh + currentLow) / 2;
					directionalPivot	= 2*pp - centralPivot;
					if (volSum > double.Epsilon)
						previousVWAP	= currentVolPriceSum/currentVolSum;
					else
						previousVWAP	= 0;
					if (noDailyData)
						plotPivots = false;
					else
						plotPivots = true;
				}	
				if (priorDayHLC != HLCCalculationMode.UserDefinedValues)
				{
					if (!existsHistDailyData || (existsHistDailyData && rthPivots)) 
					{
						if (numberOfSessions == 1 || !rthPivots || (rthPivots && activeSession == anaSessionCountPD38.First))
						{
							if (!existsHistDailyData)	
								currentClose		= close;
							if (!bridgePublicHoliday || rthPivots)
							{
								currentHigh		= high;
								currentLow		= low;
								volSum			= 0;
								volPriceSum		= 0;
							}
							else
							{
								currentHigh		= Math.Max(currentHigh, high);
								currentLow		= Math.Min(currentLow, low);
								volSum			= currentVolSum;
								volPriceSum		= currentVolPriceSum;
							}
							currentVolSum		= volSum + Volume[0];
							currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
							periodOpen 			= true;
						}
						else 
						{	
							volSum				= 0;
							volPriceSum			= 0;
							currentVolPriceSum 	= 0;
							currentVolSum		= 0;
							periodOpen			= false;
						}
					}
					else  
					{	
						if (!bridgePublicHoliday || noCurrentDailyBar)
						{
							currentHigh 	= double.MinValue;
							currentLow 		= double.MaxValue;
						}
						if (!bridgePublicHoliday)
						{
							volSum			= 0;
							volPriceSum		= 0;
						}
						else
						{
							volSum			= currentVolSum;
							volPriceSum		= currentVolPriceSum;
						}
						currentVolSum		= volSum + Volume[0];
						currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
						periodOpen 			= true;
					}
				}
			}
			else if (priorDayHLC != HLCCalculationMode.UserDefinedValues && Bars.FirstBarOfSession)
			{
				if (FirstTickOfBar)
				{	
					sessionCount = sessionCount + 1;
					numberOfSessions = Math.Min(3, Math.Max(sessionCount, numberOfSessions));
				}
				if (existsHistDailyData && !rthPivots) 
				{
					if (FirstTickOfBar)
					{
						volSum			= currentVolSum;
						volPriceSum		= currentVolPriceSum;
					}
					currentVolSum		= volSum + Volume[0];
					currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
					periodOpen			= true;
				}	
				else if (!existsHistDailyData && !rthPivots)
				{
					currentClose	= close;
					currentHigh		= Math.Max(currentHigh, high);
					currentLow		= Math.Min(currentLow, low);
					if (FirstTickOfBar)
					{
						volSum			= currentVolSum;
						volPriceSum		= currentVolPriceSum;
					}
					currentVolSum		= volSum + Volume[0];
					currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
					periodOpen			= true;
				}
				else if ((sessionCount == 1 && activeSession == anaSessionCountPD38.First)|| (sessionCount == 2 && 
					activeSession == anaSessionCountPD38.Second) || (sessionCount == 3 && activeSession == anaSessionCountPD38.Third))
				{
					if (FirstTickOfBar)
					{
						volSum				= 0;
						volPriceSum			= 0;
					}
					if (!existsHistDailyData)	
						currentClose	= close;
					currentHigh			= high;
					currentLow			= low;
					currentVolSum		= Volume[0];
					currentVolPriceSum	= Typical[0]*Volume[0];
					periodOpen			= true;
				}
				else 
					periodOpen = false;
			}
			else if (priorDayHLC != HLCCalculationMode.UserDefinedValues && periodOpen)
			{	
				if (!existsHistDailyData || (existsHistDailyData && rthPivots)) 
				{
					if (!existsHistDailyData)
						currentClose	= close;
					currentHigh		= Math.Max(currentHigh, high);
					currentLow		= Math.Min(currentLow, low);
				}
				if (FirstTickOfBar)
				{
					volSum			= currentVolSum;
					volPriceSum		= currentVolPriceSum;
				}
				currentVolSum		= volSum + Volume[0];
				currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
			}
			
			if (plotPivots && currentDate != Cbi.Globals.MinDate && !(rthPivots && activeSession == anaSessionCountPD38.Third && numberOfSessions == 2))
			{
				pp = Math.Round(pp/displaySize)* displaySize;
				r1 = Math.Round(r1/displaySize)* displaySize;
				s1 = Math.Round(s1/displaySize)* displaySize;
				r2 = Math.Round(r2/displaySize)* displaySize;
				s2 = Math.Round(s2/displaySize)* displaySize;
				r3 = Math.Round(r3/displaySize)* displaySize;
				s3 = Math.Round(s3/displaySize)* displaySize;
				r4 = Math.Round(r4/displaySize)* displaySize;
				s4 = Math.Round(s4/displaySize)* displaySize;
				r5 = Math.Round(r5/displaySize)* displaySize;
				s5 = Math.Round(s5/displaySize)* displaySize;
				PP.Set(pp);
				R1.Set(r1);
				S1.Set(s1);
				R2.Set(r2);
				S2.Set(s2);
				R3.Set(r3);
				S3.Set(s3);
				R4.Set(r4);
				S4.Set(s4);
				R5.Set(r5);
				S5.Set(s5);
				if (showMidpivots && PivotFormula != anaPivotStylesPD38.Camarilla && 
					PivotFormula != anaPivotStylesPD38.Fibonacci && PivotFormula != anaPivotStylesPD38.Jackson)
				{
					rmid = Math.Round(rmid/displaySize)* displaySize;
					smid = Math.Round(smid/displaySize)* displaySize;
					r12mid = Math.Round(r12mid/displaySize)* displaySize;
					s12mid = Math.Round(s12mid/displaySize)* displaySize;
					r23mid = Math.Round(r23mid/displaySize)* displaySize;
					s23mid = Math.Round(s23mid/displaySize)* displaySize;
					r34mid = Math.Round(r34mid/displaySize)* displaySize;
					s34mid = Math.Round(s34mid/displaySize)* displaySize;
					r45mid = Math.Round(r45mid/displaySize)* displaySize;
					s45mid = Math.Round(s45mid/displaySize)* displaySize;
					RMid.Set(rmid);
					SMid.Set(smid);
					R12Mid.Set(r12mid);
					S12Mid.Set(s12mid);
					R23Mid.Set(r23mid);
					S23Mid.Set(s23mid);
					R34Mid.Set(r34mid);
					S34Mid.Set(s34mid);
					R45Mid.Set(r45mid);
					S45Mid.Set(s45mid);
				}	
				centralPivot = Math.Round(centralPivot/displaySize)* displaySize;
				directionalPivot = Math.Round(directionalPivot/displaySize)* displaySize;
				PreviousHigh.Set(previousHigh);
				PreviousLow.Set(previousLow);
				PreviousClose.Set(previousClose);
				CentralPivot.Set(centralPivot);
				DirectionalPivot.Set(directionalPivot);
				if (Instrument.MasterInstrument.InstrumentType != Cbi.InstrumentType.Index && 
				 	Instrument.MasterInstrument.InstrumentType != Cbi.InstrumentType.Currency && previousVWAP > double.Epsilon
					&& priorDayHLC != HLCCalculationMode.UserDefinedValues)
				{	
					previousVWAP = Math.Round(previousVWAP/displaySize)* displaySize;
					PVWAP.Set(previousVWAP);
				}
				else
					PVWAP.Reset();
			}
			else
			{	
				PP.Set(0);
				R1.Set(0);
				S1.Set(0);
				R2.Set(0);
				S2.Set(0);
				R3.Set(0);
				S3.Set(0);
				R4.Set(0);
				S4.Set(0);
				R5.Set(0);
				S5.Set(0);
				RMid.Set(0);
				SMid.Set(0);
				R12Mid.Set(0);
				S12Mid.Set(0);
				R23Mid.Set(0);
				S23Mid.Set(0);
				R34Mid.Set(0);
				S34Mid.Set(0);
				R45Mid.Set(0);
				S45Mid.Set(0);
				PreviousHigh.Set(0);
				PreviousLow.Set(0);
				PreviousClose.Set(0);
				CentralPivot.Set(0);
				DirectionalPivot.Set(0);
				PVWAP.Set(0);
			}
			if (existsHistDailyData && (rthPivots || priorDayHLC == HLCCalculationMode.UserDefinedValues) 
				&& (PreviousClose[0] < PreviousLow[0] || PreviousClose[0] > PreviousHigh[0]))
				DrawTextFixed("errortag5", errorData5, TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
			else
				DrawTextFixed("errortag5", "", TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
		}

		#region Properties

		/// <summary>
		/// </summary>
		[Description("Select the settlement or close price used for calculating pivots.")]
		[GridCategory("Options")]
		[Gui.Design.DisplayNameAttribute("Settlement/Close")]
		public Data.HLCCalculationMode PriorDayHLC
		{
			get { return priorDayHLC; }
			set { priorDayHLC = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Option to show MidPivots")]
		[GridCategory("Options")]
		[Gui.Design.DisplayNameAttribute("Show Midpivots")]
		public bool ShowMidpivots 
		{
			get { return showMidpivots; }
			set { showMidpivots = value; }
		}
		

		/// <summary>
		/// </summary>
		[Description("Pivot Formula Type")]
		[GridCategory("Options")]
		[Gui.Design.DisplayNameAttribute("Pivot Formula")]
		public anaPivotStylesPD38 PivotFormula
		{
			get { return pivotFormula; }
			set { pivotFormula = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Session used for calculating pivots")]
		[GridCategory("Options")]
		[Gui.Design.DisplayNameAttribute("Session")]
		public anaSessionTypePD38 PivotSession
		{
			get { return pivotSession; }
			set { pivotSession = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Session used for calculating RTH pivots")]
		[GridCategory("Options")]
		[Gui.Design.DisplayNameAttribute("Session # for RTH")]
		public anaSessionCountPD38 ActiveSession
		{
			get { return activeSession; }
			set { activeSession = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PP
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R1
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S1
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R2
		{
			get { return Values[3]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S2
		{
			get { return Values[4]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R3
		{
			get { return Values[5]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S3
		{
			get { return Values[6]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R4
		{
			get { return Values[7]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S4
		{
			get { return Values[8]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R5
		{
			get { return Values[9]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S5
		{
			get { return Values[10]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries RMid
		{
			get { return Values[11]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries SMid
		{
			get { return Values[12]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R12Mid
		{
			get { return Values[13]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S12Mid
		{
			get { return Values[14]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R23Mid
		{
			get { return Values[15]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S23Mid
		{
			get { return Values[16]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R34Mid
		{
			get { return Values[17]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S34Mid
		{
			get { return Values[18]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R45Mid
		{
			get { return Values[19]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S45Mid
		{
			get { return Values[20]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousHigh
		{
			get { return Values[21]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousLow
		{
			get { return Values[22]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousClose
		{
			get { return Values[23]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries CentralPivot
		{
			get { return Values[24]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries DirectionalPivot
		{
			get { return Values[25]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PVWAP
		{
			get { return Values[26]; }
		}

		/// <summary>
		/// close value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session close value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedClose
		{
			get { return userDefinedClose; }
			set { userDefinedClose = value; }
		}

		/// <summary>
		/// high value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session high value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedHigh
		{
			get { return userDefinedHigh; }
			set { userDefinedHigh = value; }
		}

		/// <summary>
		/// low value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session low value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedLow
		{
			get { return userDefinedLow; }
			set { userDefinedLow = value; }
		}

		/// <summary>
		/// </summary>
		[Description("Option where to plot labels")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Label Position")]
		public anaPlotAlignPD38 PlotLabels
		{
			get { return plotLabels; }
			set { plotLabels = value; }
		}
			
		/// <summary>
		/// </summary>
		[Description("Label distance from line.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Label Offset")]
		public int LabelOffset
		{
			get { return labelOffset; }
			set { labelOffset = Math.Max(1, value); }
		}
	
		/// <summary>
		/// </summary>
		[Description("Font size for labels.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Label Font Size")]
		public int LabelFontSize
		{
			get { return labelFontSize; }
			set { labelFontSize = Math.Max(1, value); }
		}
	
		/// <summary>
		/// </summary>
		[Description("Length of the plots as # of bars.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Max. Plot Length # Bars")]
		public int Width
		{
			get { return width; }
			set { width = Math.Max(0, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Width for Pivots.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Line Width Pivot")]
		public int Plot0Width
		{
			get { return plot0Width; }
			set { plot0Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for Pivots.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash Style Pivots")]
		public DashStyle Dash0Style
		{
			get { return dash0Style; }
			set { dash0Style = value; }
		} 

		/// <summary>
		/// </summary>
		[Description("Width for Midpivots.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Line Width Midpivots")]
		public int Plot1Width
		{
			get { return plot1Width; }
			set { plot1Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for Midpivots.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash Style Midpivots")]
		public DashStyle Dash1Style
		{
			get { return dash1Style; }
			set { dash1Style = value; }
		} 
		
		/// </summary>
		[Description("Width for High, Low and Close.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Line Width High, Low, Close")]
		public int Plot2Width
		{
			get { return plot2Width; }
			set { plot2Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for High, Low and Close.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash Style High, Low, Close")]
		public DashStyle Dash2Style
		{
			get { return dash2Style; }
			set { dash2Style = value; }
		} 
		
		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Main Pivot")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Main Pivot")]
		public Color PivotColor
		{
			get { return pivotColor; }
			set { pivotColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string PivotColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(pivotColor); }
			set { pivotColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Resistance Pivots")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Resistance")]
		public Color ResistanceColor
		{
			get { return resistanceColor; }
			set { resistanceColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string ResistanceColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(resistanceColor); }
			set { resistanceColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Support Pivots")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Support")]
		public Color SupportColor
		{
			get { return supportColor; }
			set { supportColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string SupportColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(supportColor); }
			set { supportColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Midpivots")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Midpivots")]
		public Color MidpivotColor
		{
			get { return midpivotColor; }
			set { midpivotColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string MidpivotColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(midpivotColor); }
			set { midpivotColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Prior High")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Prior High")]
		public Color HighColor
		{
			get { return highColor; }
			set { highColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string HighColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(highColor); }
			set { highColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Prior Low")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Prior Low")]
		public Color LowColor
		{
			get { return lowColor; }
			set { lowColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string LowColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(lowColor); }
			set { lowColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Prior Close")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Prior Close")]
		public Color CloseColor
		{
			get { return closeColor; }
			set { closeColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string CloseColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(closeColor); }
			set { closeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Central Pivot")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Central Pivot")]
		public Color CentralPivotColor
		{
			get { return centralPivotColor; }
			set { centralPivotColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string CentralPivotColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(centralPivotColor); }
			set { centralPivotColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Directional Pivot")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Directional Pivot")]
		public Color DirectionalPivotColor
		{
			get { return directionalPivotColor; }
			set { directionalPivotColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DirectionalPivotColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(directionalPivotColor); }
			set { directionalPivotColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore]
		[Description("Select Color for Prior VWAP")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Prior VWAP")]
		public Color VWAPColor
		{
			get { return vwapColor; }
			set { vwapColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string VWAPColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(vwapColor); }
			set { vwapColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
        [Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 01 /No Trade Date")]
		public DateTime PublicHoliday0
		{
			get { return publicHoliday0;}
			set { publicHoliday0 = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
        [Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 02 /No Trade Date")]
		public DateTime PublicHoliday1
		{
			get { return publicHoliday1;}
			set { publicHoliday1 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 03 /No Trade Date")]
		public DateTime PublicHoliday2
		{
			get { return publicHoliday2;}
			set { publicHoliday2 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 04 /No Trade Date")]
		public DateTime PublicHoliday3
		{
			get { return publicHoliday3;}
			set { publicHoliday3 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 05 /No Trade Date")]
		public DateTime PublicHoliday4
		{
			get { return publicHoliday4;}
			set { publicHoliday4 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 06 /No Trade Date")]
		public DateTime PublicHoliday5
		{
			get { return publicHoliday5;}
			set { publicHoliday5 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 07 /No Trade Date")]
		public DateTime PublicHoliday6
		{
			get { return publicHoliday6;}
			set { publicHoliday6 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 08 /No Trade Date")]
		public DateTime PublicHoliday7
		{
			get { return publicHoliday7;}
			set { publicHoliday7 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 09 /No Trade Date")]
		public DateTime PublicHoliday8
		{
			get { return publicHoliday8;}
			set { publicHoliday8 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 10 /No Trade Date")]
		public DateTime PublicHoliday9
		{
			get { return publicHoliday9;}
			set { publicHoliday9 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 11 /No Trade Date")]
		public DateTime PublicHoliday10
		{
			get { return publicHoliday10;}
			set { publicHoliday10 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 12 /No Trade Date")]
		public DateTime PublicHoliday11
		{
			get { return publicHoliday11;}
			set { publicHoliday11 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 13 /No Trade Date")]
		public DateTime PublicHoliday12
		{
			get { return publicHoliday12;}
			set { publicHoliday12 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 14 /No Trade Date")]
		public DateTime PublicHoliday13
		{
			get { return publicHoliday13;}
			set { publicHoliday13 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 15 /No Trade Date")]
		public DateTime PublicHoliday14
		{
			get { return publicHoliday14;}
			set { publicHoliday14 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 16 /No Trade Date")]
		public DateTime PublicHoliday15
		{
			get { return publicHoliday15;}
			set { publicHoliday15 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 17 /No Trade Date")]
		public DateTime PublicHoliday16
		{
			get { return publicHoliday16;}
			set { publicHoliday16 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 18 /No Trade Date")]
		public DateTime PublicHoliday17
		{
			get { return publicHoliday17;}
			set { publicHoliday17 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 19 /No Trade Date")]
		public DateTime PublicHoliday18
		{
			get { return publicHoliday18;}
			set { publicHoliday18 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 20 /No Trade Date")]
		public DateTime PublicHoliday19
		{
			get { return publicHoliday19;}
			set { publicHoliday19 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 21 /No Trade Date")]
		public DateTime PublicHoliday20
		{
			get { return publicHoliday20;}
			set { publicHoliday20 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 22 /No Trade Date")]
		public DateTime PublicHoliday21
		{
			get { return publicHoliday21;}
			set { publicHoliday21 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 23 /No Trade Date")]
		public DateTime PublicHoliday22
		{
			get { return publicHoliday22;}
			set { publicHoliday22 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Settlement Next Day For Globex")]
		[Gui.Design.DisplayNameAttribute("Holiday 24 /No Trade Date")]
		public DateTime PublicHoliday23
		{
			get { return publicHoliday23;}
			set { publicHoliday23 = value;}
		}		
		#endregion

		#region Miscellaneous

		public override string FormatPriceMarker(double price)
		{
			double trunc = Math.Truncate(price);
			int fraction = Convert.ToInt32(320 * Math.Abs(price - trunc) - 0.0001); // rounding down for ZF and ZT
			string priceMarker = "";
			if (TickSize == 0.03125) 
			{
				fraction = fraction/10;
				if (fraction < 10)
					priceMarker = trunc.ToString() + "'0" + fraction.ToString();
				else 
					priceMarker = trunc.ToString() + "'" + fraction.ToString();
			}
			else if (TickSize == 0.015625 || TickSize == 0.0078125)
			{
				if (fraction < 10)
					priceMarker = trunc.ToString() + "'00" + fraction.ToString();
				else if (fraction < 100)
					priceMarker = trunc.ToString() + "'0" + fraction.ToString();
				else	
					priceMarker = trunc.ToString() + "'" + fraction.ToString();
			}
			else
				priceMarker = price.ToString(Gui.Globals.GetTickFormatString(TickSize));
			return priceMarker;
		}		

		/// <summary>
		/// Have the .GetBars() call in a seperate thread. Reason: NT internal pool locking would not work if called in main thread.
		/// </summary>
		/// <param name="state"></param>
		private void GetBarsNow(object state)
		{
			if (Disposed)
				return;

			dailyBars			= Data.Bars.GetBars(Bars.Instrument, new Period(PeriodType.Day, 1, Bars.Period.MarketDataType), Bars.From, Bars.To, (Session) Bars.Session.Clone(), Data.Bars.SplitAdjust, Data.Bars.DividendAdjust);
			existsHistDailyData	= (dailyBars.Count <= 1) ? false : true;
			isDailyDataLoaded	= true;
			Enabled				= true;

			Cbi.Globals.SynchronizeInvoke.AsyncInvoke(new System.Windows.Forms.MethodInvoker(InvalidateNow), null);
		}

		private DateTime GetLastBarSessionDate(DateTime time, Data.Bars bars, int barsAgo, PivotRange pivotRange)
		{
			if (time > cacheSessionEndTmp)
			{	
				extendPublicHoliday = false;
				for (int i =0; i<24; i++)
				{
					if (publicHoliday[i].Date == sessionDateTmp)
						extendPublicHoliday = true;
				}
				if (Bars.BarsType.IsIntraday)
					sessionDateTmp = Bars.GetTradingDayFromLocal(time);
				else
					sessionDateTmp = time.Date;
				if(cacheSessionDate != sessionDateTmp) 
				{
					cacheSessionDate = sessionDateTmp;
					if (!extendPublicHoliday && (newSessionBarIdxArr1.Count == 0 
						|| (newSessionBarIdxArr1.Count > 0 && CurrentBar > (int) newSessionBarIdxArr1[newSessionBarIdxArr1.Count - 1])))
							newSessionBarIdxArr1.Add(CurrentBar);
				}
				Bars.Session.GetNextBeginEnd(bars, barsAgo, out cacheSessionBeginTmp, out cacheSessionEndTmp);
			}
			return sessionDateTmp;
		}

		private DateTime RoundUpTimeToPeriodTime(DateTime time, PivotRange pivotRange)
		{
			if (pivotRange == PivotRange.Weekly)
			{
				DateTime periodStart = time.AddDays((6 - (((int) time.DayOfWeek) + 1) % 7));
				return periodStart.Date.AddDays(System.Math.Ceiling(System.Math.Ceiling(time.Date.Subtract(periodStart.Date).TotalDays) / 7) * 7).Date;
			}
			else if (pivotRange == PivotRange.Monthly)
			{
				DateTime result = new DateTime(time.Year, time.Month, 1); 
				return result.AddMonths(1).AddDays(-1);
			}
			else
				return time;
		}

		internal void InvalidateNow()
		{
			if (Disposed || ChartControl == null)
				return;

			ChartControl.Invalidate(true);
		}

		/// <summary>
        /// Overload this method to handle the termination of an indicator. Use this method to dispose of any resources vs overloading the Dispose() method.
		/// </summary>
		protected override void OnTermination()
		{
			if (dailyBars != null)
				dailyBars.Dispose();

			textBrush.Dispose();
			foreach (SolidBrush solidBrush in brushes)
				solidBrush.Dispose();
			stringFormatCenter.Dispose();
			stringFormatFar.Dispose();	
			stringFormatNear.Dispose();
		}
	
		/// <summary>
		/// </summary>
		/// <param name="graphics"></param>
		/// <param name="bounds"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null || ChartControl == null)
				return;
			// plot error if data not complete
			DateTime lastBarSessionBegin = Cbi.Globals.MinDate;
			DateTime lastBarSessionEnd = Cbi.Globals.MinDate;
			if (ChartControl.EquidistantBars)
				Bars.Session.GetNextBeginEnd(Bars.Get(Math.Min(Bars.Count - 1, ChartControl.LastBarPainted)).Time, out lastBarSessionBegin, out lastBarSessionEnd);
			else
				Bars.Session.GetNextBeginEnd(Bars.Get(Math.Min(Bars.Count - 1, Bars.GetBar(ChartControl.LastBarTimePainted))).Time, out lastBarSessionBegin, out lastBarSessionEnd);
			textBrush.Color = ChartControl.AxisColor;
			SizeF errorSize	= graphics.MeasureString(errorData4, textFont);
			errorTextWidth	= errorSize.Width + 5;
			errorTextHeight	= errorSize.Height + 5;
			if (Bars.Count > 0 && plotStart >= lastBarSessionEnd && priorDayHLC != HLCCalculationMode.UserDefinedValues)
				graphics.DrawString(errorData4, textFont, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);

			int	firstBarIdxToPaint	= -1;
			for (int i = newSessionBarIdxArr1.Count - 1; i >= 0; i--)
			{
				int prevSessionBreakIdx = (int) newSessionBarIdxArr1[i];
				if (prevSessionBreakIdx <= this.LastBarIndexPainted)
				{
					firstBarIdxToPaint = prevSessionBreakIdx;
					break;
				}
			}

			int lastBarIndex = this.LastBarIndexPainted + ((CalculateOnBarClose && this.LastBarIndexPainted == Input.Count - 2) ? 1 : 0);
			for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
			{
				SolidBrush		brush				= brushes[seriesCount];
				int				lastX				= -1;
				int				lastY				= -1;
				int				firstX				= -1;
				SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
				Gui.Chart.Plot	plot				= Plots[seriesCount];
				DataSeries		series				= (DataSeries) Values[seriesCount];

				using (GraphicsPath	path = new GraphicsPath()) 
				{
					if (brush.Color != plot.Pen.Color)	
						brush = new SolidBrush(plot.Pen.Color);

					for (int idx = lastBarIndex; idx >= Math.Max(this.FirstBarIndexPainted, lastBarIndex - Width); idx--)
					{
						if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
							continue;
						else if (!series.IsValidPlot(this.LastBarIndexPainted))
							continue;

						if (idx < firstBarIdxToPaint)
							break;
						
						double  val = series.Get(this.LastBarIndexPainted);
						int		x	= ChartControl.GetXByBarIdx(BarsArray[0], idx);
						int		y	= ChartControl.GetYByValue(this, val);

						if (lastX >= 0)
						{
							if (y != lastY) // Problem here is, that last bar of old day has date of new day
								y = lastY;
							path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
						lastX	= x;
						lastY	= y;
						if (idx == lastBarIndex)
							firstX	= x;
					}

					graphics.SmoothingMode = SmoothingMode.AntiAlias;
					graphics.DrawPath(plot.Pen, path);
					graphics.SmoothingMode = oldSmoothingMode;
					if (PlotLabels == anaPlotAlignPD38.Right)
					graphics.DrawString(plot.Name, labelFont, brush, firstX + LabelOffset, lastY - labelFont.GetHeight() / 2, stringFormatNear);
					if (PlotLabels == anaPlotAlignPD38.Left)
					graphics.DrawString(plot.Name, labelFont, brush, lastX - LabelOffset, lastY - labelFont.GetHeight() / 2, stringFormatFar);
				}
			}
		}
		#endregion
	}
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaPivotsDailyV38[] cacheanaPivotsDailyV38 = null;

        private static anaPivotsDailyV38 checkanaPivotsDailyV38 = new anaPivotsDailyV38();

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public anaPivotsDailyV38 anaPivotsDailyV38(anaSessionCountPD38 activeSession, anaPivotStylesPD38 pivotFormula, anaSessionTypePD38 pivotSession, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow)
        {
            return anaPivotsDailyV38(Input, activeSession, pivotFormula, pivotSession, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow);
        }

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public anaPivotsDailyV38 anaPivotsDailyV38(Data.IDataSeries input, anaSessionCountPD38 activeSession, anaPivotStylesPD38 pivotFormula, anaSessionTypePD38 pivotSession, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow)
        {
            if (cacheanaPivotsDailyV38 != null)
                for (int idx = 0; idx < cacheanaPivotsDailyV38.Length; idx++)
                    if (cacheanaPivotsDailyV38[idx].ActiveSession == activeSession && cacheanaPivotsDailyV38[idx].PivotFormula == pivotFormula && cacheanaPivotsDailyV38[idx].PivotSession == pivotSession && cacheanaPivotsDailyV38[idx].PriorDayHLC == priorDayHLC && cacheanaPivotsDailyV38[idx].ShowMidpivots == showMidpivots && Math.Abs(cacheanaPivotsDailyV38[idx].UserDefinedClose - userDefinedClose) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV38[idx].UserDefinedHigh - userDefinedHigh) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV38[idx].UserDefinedLow - userDefinedLow) <= double.Epsilon && cacheanaPivotsDailyV38[idx].EqualsInput(input))
                        return cacheanaPivotsDailyV38[idx];

            lock (checkanaPivotsDailyV38)
            {
                checkanaPivotsDailyV38.ActiveSession = activeSession;
                activeSession = checkanaPivotsDailyV38.ActiveSession;
                checkanaPivotsDailyV38.PivotFormula = pivotFormula;
                pivotFormula = checkanaPivotsDailyV38.PivotFormula;
                checkanaPivotsDailyV38.PivotSession = pivotSession;
                pivotSession = checkanaPivotsDailyV38.PivotSession;
                checkanaPivotsDailyV38.PriorDayHLC = priorDayHLC;
                priorDayHLC = checkanaPivotsDailyV38.PriorDayHLC;
                checkanaPivotsDailyV38.ShowMidpivots = showMidpivots;
                showMidpivots = checkanaPivotsDailyV38.ShowMidpivots;
                checkanaPivotsDailyV38.UserDefinedClose = userDefinedClose;
                userDefinedClose = checkanaPivotsDailyV38.UserDefinedClose;
                checkanaPivotsDailyV38.UserDefinedHigh = userDefinedHigh;
                userDefinedHigh = checkanaPivotsDailyV38.UserDefinedHigh;
                checkanaPivotsDailyV38.UserDefinedLow = userDefinedLow;
                userDefinedLow = checkanaPivotsDailyV38.UserDefinedLow;

                if (cacheanaPivotsDailyV38 != null)
                    for (int idx = 0; idx < cacheanaPivotsDailyV38.Length; idx++)
                        if (cacheanaPivotsDailyV38[idx].ActiveSession == activeSession && cacheanaPivotsDailyV38[idx].PivotFormula == pivotFormula && cacheanaPivotsDailyV38[idx].PivotSession == pivotSession && cacheanaPivotsDailyV38[idx].PriorDayHLC == priorDayHLC && cacheanaPivotsDailyV38[idx].ShowMidpivots == showMidpivots && Math.Abs(cacheanaPivotsDailyV38[idx].UserDefinedClose - userDefinedClose) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV38[idx].UserDefinedHigh - userDefinedHigh) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV38[idx].UserDefinedLow - userDefinedLow) <= double.Epsilon && cacheanaPivotsDailyV38[idx].EqualsInput(input))
                            return cacheanaPivotsDailyV38[idx];

                anaPivotsDailyV38 indicator = new anaPivotsDailyV38();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ActiveSession = activeSession;
                indicator.PivotFormula = pivotFormula;
                indicator.PivotSession = pivotSession;
                indicator.PriorDayHLC = priorDayHLC;
                indicator.ShowMidpivots = showMidpivots;
                indicator.UserDefinedClose = userDefinedClose;
                indicator.UserDefinedHigh = userDefinedHigh;
                indicator.UserDefinedLow = userDefinedLow;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaPivotsDailyV38[] tmp = new anaPivotsDailyV38[cacheanaPivotsDailyV38 == null ? 1 : cacheanaPivotsDailyV38.Length + 1];
                if (cacheanaPivotsDailyV38 != null)
                    cacheanaPivotsDailyV38.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaPivotsDailyV38 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaPivotsDailyV38 anaPivotsDailyV38(anaSessionCountPD38 activeSession, anaPivotStylesPD38 pivotFormula, anaSessionTypePD38 pivotSession, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow)
        {
            return _indicator.anaPivotsDailyV38(Input, activeSession, pivotFormula, pivotSession, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow);
        }

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaPivotsDailyV38 anaPivotsDailyV38(Data.IDataSeries input, anaSessionCountPD38 activeSession, anaPivotStylesPD38 pivotFormula, anaSessionTypePD38 pivotSession, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow)
        {
            return _indicator.anaPivotsDailyV38(input, activeSession, pivotFormula, pivotSession, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaPivotsDailyV38 anaPivotsDailyV38(anaSessionCountPD38 activeSession, anaPivotStylesPD38 pivotFormula, anaSessionTypePD38 pivotSession, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow)
        {
            return _indicator.anaPivotsDailyV38(Input, activeSession, pivotFormula, pivotSession, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow);
        }

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaPivotsDailyV38 anaPivotsDailyV38(Data.IDataSeries input, anaSessionCountPD38 activeSession, anaPivotStylesPD38 pivotFormula, anaSessionTypePD38 pivotSession, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaPivotsDailyV38(input, activeSession, pivotFormula, pivotSession, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow);
        }
    }
}
#endregion
